@@ -31,6 +31,7 @@ using Gtk;
 using GConf;
 using Mono.Unix;
 using Avahi;
+using NDesk.DBus;
 
 using GShare.FTP;
 
@@ -39,8 +40,7 @@ namespace GShare
     public class GShared : IGShareDaemon
     {
         private readonly string 
-            HOME_SHARE = String.Format(Catalog.GetString("{0}/Shared Files"),
-                                        Environment.GetFolderPath(Environment.SpecialFolder.Personal));
+            HOME_SHARE = Catalog.GetString("~/Shared Files");
 
         private const string
             AVAHI_DOMAIN = "local",
@@ -57,8 +57,11 @@ namespace GShare
             GCONF_USERNAME = "/apps/gshare/username",
             GCONF_PASSWORD = "/apps/gshare/password",
             GCONF_FIRST_TIME = "/apps/gshare/first_time",
-            GCONF_SHARE_DIR = "/apps/gshare/share_dir";
+            GCONF_SHARE_DIR = "/apps/gshare/share_dir",
+            GCONF_PORT = "/apps/gshare/port",
+            GCONF_SHARE_NAME = "/apps/gshare/share_name";
             
+        private const int DEFAULT_SERVER_PORT = 10021;            
 
         private FTPServer iFtpServer = null;
         private GShareService iDbusService = null;
@@ -72,6 +75,52 @@ namespace GShare
         private bool iDaemonRunning = false;
 
         /* properties */
+        public string ShareName
+        {
+            get
+            {
+                string shareName = "";
+                try
+                {
+                    shareName = (string)iGconf.Get(GCONF_SHARE_NAME);
+                }
+                catch(GConf.NoSuchKeyException ex)
+                {
+                }
+
+                if (shareName.Trim() == "")
+                {
+                    string username = null;
+                    UnixUserInfo user = Mono.Unix.UnixUserInfo.GetRealUser();
+                    if (user.RealName != null && user.RealName.Length > 0)
+                        username = user.RealName.Split(new char[]{','})[0];
+
+                    if (username == null || username == "")
+                    {
+                        /*this happens on ArchLinux*/
+                        shareName = String.Format(AVAHI_SERVICE_NAME, Mono.Unix.UnixEnvironment.UserName);
+                    }
+                    else
+                        shareName = String.Format(AVAHI_SERVICE_NAME, username);
+
+                    iGconf.Set(GCONF_SHARE_NAME,shareName);
+                }
+
+                return shareName;
+
+            }
+            set 
+            {
+                string oldValue = ShareName;
+                iGconf.Set(GCONF_SHARE_NAME,value); 
+                if (oldValue != value)
+                {
+                    iAvahiServiceName = value;
+                    StopZeroConf();
+                    StartZeroConf();
+                }
+            }
+        }
         public bool IsFirstTime
         {
             get
@@ -114,17 +163,18 @@ namespace GShare
 
             set 
             {
-            
-                iGconf.Set(GCONF_SHARE_ENABLED, value);
-                
+
+                bool reallyEnable = value;
                 if (value)
-                    StartDaemon();
+                    reallyEnable = StartDaemon();
                 else
                     StopDaemon();
                 
+                iGconf.Set(GCONF_SHARE_ENABLED, reallyEnable);
+                
                 /* if sharing is enabled, autostart the service */
-                if (Utility.GetGnomeVersion() >= 2.14)
-                    StartAfterLogin = value;
+                if (Utility.GnomeVersion >= 2.14)
+                    StartAfterLogin = reallyEnable;
                 
             }
         }
@@ -147,6 +197,36 @@ namespace GShare
             set { iGconf.Set(GCONF_ALLOW_ANONYMOUS, value); }
         }
 
+        public int ServerPort
+        {
+            get
+            {
+                try
+                {
+                    return (int)iGconf.Get(GCONF_PORT);
+                }
+                catch(GConf.NoSuchKeyException ex)
+                {
+                    iGconf.Set(GCONF_PORT,DEFAULT_SERVER_PORT);
+                    return DEFAULT_SERVER_PORT;
+                }
+            }
+            set 
+            { 
+                int oldValue = ServerPort;
+                iGconf.Set(GCONF_PORT,value); 
+                if (oldValue != value)
+                {
+                    /* 
+                        just restart the whole daemon instead of just 
+                        restarting the FTP server and the ZeroConf client 
+                    */
+                    StopDaemon();
+                    StartDaemon();
+                }
+            }
+        }
+
         public bool AllowWrite
         {
             get
@@ -225,24 +305,15 @@ namespace GShare
         {
             get 
             {
-                return HOME_SHARE;
-                /*
-                    This is disabled because I'm failing to understand how to
-                    convert ~ to /home/<user> automatically. 
-                    There has to be a class that does this conversion,
-                    but I cannot find out which one.
-                */
-                /*
                 try
                 {
-                    return (string)iGconf.Get(GCONF_SHARE_DIR);
+                    return Utility.GetCanonicalShareRootPath( (string)iGconf.Get(GCONF_SHARE_DIR) );
                 }
                 catch(GConf.NoSuchKeyException ex)
                 {
                     iGconf.Set(GCONF_SHARE_DIR,HOME_SHARE);
-                    return HOME_SHARE;
+                    return Utility.GetCanonicalShareRootPath(HOME_SHARE);
                 }
-                */
             }
         }
 
@@ -250,7 +321,7 @@ namespace GShare
         {
             get 
             {
-                return File.Exists(String.Concat(Utility.GetXdgAutoStartDir(),
+                return File.Exists(String.Concat(Utility.XdgAutoStartDir,
                                                     Constants.AUTOSTART_FILE));
             }
             set 
@@ -258,8 +329,8 @@ namespace GShare
                 if (value)
                 {
                      
-                    string xdgDir = Utility.GetXdgAutoStartDir();
-                    string autostartFilePath = Utility.GetDaemonAutoStartFilePath();
+                    string xdgDir = Utility.XdgAutoStartDir;
+                    string autostartFilePath = Utility.DaemonAutoStartFilePath;
                      
                     if (File.Exists(String.Concat(xdgDir,Constants.AUTOSTART_FILE)))
                     {
@@ -285,7 +356,7 @@ namespace GShare
                  }
                  else
                  {
-                     File.Delete(String.Concat( Utility.GetXdgAutoStartDir(),
+                     File.Delete(String.Concat( Utility.XdgAutoStartDir,
                                                 Constants.AUTOSTART_FILE));
                  }
             }
@@ -298,9 +369,6 @@ namespace GShare
 
             iDbusService = GShareService.GetInstance();
 
-            iFtpServer = new FTPServer(this, this, ShareRoot);
-            iFtpServer.ShutdownEvent += OnFTPServerShutdown;
-
         }
 
         private void OnFTPServerShutdown(FTPServerShutdownReason reason)
@@ -308,11 +376,11 @@ namespace GShare
             if (reason == FTPServerShutdownReason.Error)
             {
                 Log.ErrorMessage("Shutting down FTP server because of an error");
-                Utility.InvokeLater(delegate
+                Utility.InvokeLater((GShare.Utility.BackgroundTask)(delegate
                     { 
                         Utility.ShowGenericFatalErrorDialog(); 
                         Shutdown(); 
-                    });
+                    }));
             }
         }
         
@@ -323,24 +391,6 @@ namespace GShare
                      String.Compare(password,password,true) == 0));
         }
 
-        private string GetServiceName()
-        {
-
-            string username = null;
-
-            UnixUserInfo user = Mono.Unix.UnixUserInfo.GetRealUser();
-            if (user.RealName != null && user.RealName.Length > 0)
-                username = user.RealName.Split(new char[]{','})[0];
-
-            if (username == null || username == "")
-            {
-                /*this happens on ArchLinux*/
-                return String.Format(AVAHI_SERVICE_NAME, Mono.Unix.UnixEnvironment.UserName);
-            }
-            else
-                return String.Format(AVAHI_SERVICE_NAME, username);
-        }
-
         private void PublishFTPServer()
         {
             Log.DebugMessage("PublishFTPServer() IN, daemonRunning: {0}",
@@ -349,12 +399,12 @@ namespace GShare
             if (iDaemonRunning)
                 return;
 
-            iAvahiServiceName = GetServiceName();
+            iAvahiServiceName = ShareName;
             
             iFtpServer.StartServer();
             iDaemonRunning = true;
             
-            StartZeroConf(iFtpServer.ServerPort);
+            StartZeroConf();
             
         }
 
@@ -367,24 +417,24 @@ namespace GShare
             {
                 case ClientState.Collision:
                     /*TODO figure out what to do*/
-                    Utility.InvokeLater(delegate
+                    Utility.InvokeLater((GShare.Utility.BackgroundTask)(delegate
                         { 
                             StopDaemon();
-                        });
+                        }));
                     break;
                 case ClientState.Failure:
                     Log.ErrorMessage("Zeroconf client failed to register");
-                    Utility.InvokeLater(delegate
+                    Utility.InvokeLater((GShare.Utility.BackgroundTask)(delegate
                         {
                             Utility.ShowCannotPublishServiceDialog(); 
                             Shutdown(); 
-                        });
+                        }));
                     break;
                 case ClientState.Running:
-                    Utility.InvokeLater(delegate
+                    Utility.InvokeLater((GShare.Utility.BackgroundTask)(delegate
                         {
                             PublishFTPServer(); 
-                        });
+                        }));
                     break;
             }
 
@@ -398,7 +448,7 @@ namespace GShare
             switch(args.State)
             {
                 case EntryGroupState.Collision:
-                    if (iAvahiServiceName == GetServiceName())
+                    if (iAvahiServiceName == ShareName)
                     {
                         /* just collided, append the hostname to the service name */
                         iAvahiServiceName = String.Concat(iAvahiServiceName,
@@ -410,19 +460,19 @@ namespace GShare
                         iAvahiServiceName = EntryGroup.GetAlternativeServiceName(iAvahiServiceName);
                     }
                     
-                    Utility.InvokeLater(delegate
+                    Utility.InvokeLater((GShare.Utility.BackgroundTask)(delegate
                         {
-                            StartZeroConf(iFtpServer.ServerPort);
-                        });
+                            StartZeroConf();
+                        }));
                     
                     break;
                 case EntryGroupState.Failure:
                     Log.ErrorMessage("Failed to publish FTP service on zeroconf network");
-                    Utility.InvokeLater(delegate
+                    Utility.InvokeLater((GShare.Utility.BackgroundTask)(delegate
                         {
                             Utility.ShowCannotPublishServiceDialog();
                             Shutdown(); 
-                        });
+                        }));
                     break;
 
                 case EntryGroupState.Established:
@@ -433,7 +483,7 @@ namespace GShare
 
         }
 
-        private void StartZeroConf(int networkPort)
+        private void StartZeroConf()
         {
 
             /* we should make sure that client is running */
@@ -447,11 +497,11 @@ namespace GShare
                 iEntryGroup.AddService( iAvahiServiceName, 
                                         AVAHI_PROTOCOL, 
                                         AVAHI_DOMAIN, 
-                                        (ushort)networkPort);
+                                        (ushort)ServerPort);
                                         
                 iEntryGroup.Commit();
 
-                Log.DebugMessage("Listening at {0}",networkPort);
+                Log.DebugMessage("Listening at {0}",ServerPort);
 
             }
             catch (Exception ex)
@@ -460,11 +510,11 @@ namespace GShare
                                     ex.Message,
                                     ex.StackTrace);
                                 
-                Utility.InvokeLater(delegate 
+                Utility.InvokeLater((GShare.Utility.BackgroundTask)(delegate 
                     {
                         Utility.ShowCannotPublishServiceDialog(); 
                         Shutdown(); 
-                    });
+                    }));
                 
             }
 
@@ -489,7 +539,7 @@ namespace GShare
             
         }
         
-        private void StartDaemon()
+        private bool StartDaemon()
         {
 
             /*
@@ -537,17 +587,20 @@ namespace GShare
                 iAvahiClient = null;
                 
                 Utility.ShowCannotPublishServiceDialog();
-                Shutdown();
+                /*Shutdown();*/
                 
-                return;
+                return false;
                 
             }
 
             if (iAvahiClient.State == ClientState.Running)
             {
+                iFtpServer = new FTPServer(this, this, ShareRoot,ServerPort);
+                iFtpServer.ShutdownEvent += OnFTPServerShutdown;
                 PublishFTPServer();
             }
             
+            return true;
         }
         
         private void StopDaemon()
@@ -559,6 +612,7 @@ namespace GShare
 
             StopZeroConf();
             iFtpServer.ShutdownServer(FTPServerShutdownReason.NormalTermination);
+            iFtpServer = null;
 
             /* disposing the client causes a mono dump */            
             /*iAvahiClient.Dispose();*/
@@ -568,7 +622,7 @@ namespace GShare
             
         }
 
-        private void Start()
+        private bool Start()
         {
 
             Log.DebugMessage(String.Concat(String.Concat(ShareRoot," exists: "),
@@ -584,8 +638,9 @@ namespace GShare
             iDbusService.StartPublishing();
             
             if (SharingEnabled)
-                StartDaemon();
+                return StartDaemon();
 
+            return false;
         }
 
         public void Shutdown()
@@ -617,10 +672,20 @@ namespace GShare
                 ShutdownRunningDaemon();
                 return;
             }
+            else if (Array.IndexOf(args,"--version") > -1)
+            {
+                PrintVersion();
+                return;
+            }
             
             PrintUsage();
         }
         
+        private static void PrintVersion()
+        {
+            Console.WriteLine(String.Format("GShare version {0:0.00}",Utility.GShareVersion));
+        }
+            
         private static void ShutdownRunningDaemon()
         {
             if (GShareService.ServiceRunning())
@@ -636,17 +701,19 @@ namespace GShare
         
         private static void PrintUsage()
         {
-            Console.WriteLine("Usage: gshared [--help|--shutdown]");
+            Console.WriteLine("Usage: gshared [command]");
+            Console.WriteLine("gshared is the background service that shares your files.\nWhen executing gshared with no extra arguments, the gshared daemon is started");
             Console.WriteLine("--help\t\t\tPrint this help message");
             Console.WriteLine("--shutdown\t\tShutdown a running daemon");
+            Console.WriteLine("--version\t\tPrint the GShare version");
         }
         
         [STAThread]
         public static void Main(string[] args)
         {
             Application.Init("gshared",ref args);
-
             Catalog.Init(GShare.Constants.I18N_DOMAIN,GShare.Constants.I18N_DIR);
+            BusG.Init();
             
             if (args.Length > 0)
             {
@@ -691,113 +758,4 @@ namespace GShare
     }
     
 
-    public class Utility
-    {
-        private Utility(){}
-        
-        public static double GetGnomeVersion()
-        {
-
-            string version = Gnome.Modules.LibGnome.Version;
-            string[] nrs = version.Split(new char[]{'.'});
-            double res = 0d;
-
-            if (nrs.Length > 0)
-                res = Int32.Parse(nrs[0]);
-
-            if (nrs.Length > 1)
-                res += (Double.Parse(nrs[1])/100);
-            return res;
-        }
-        
-        public static string GetDaemonAutoStartFilePath()
-        {
-            string dir = Constants.GSHARE_AUTOSTART_DIR;
-            
-            if (!dir.EndsWith("/"))
-                dir = String.Concat(dir,"/");
-              
-            return String.Concat(dir,Constants.AUTOSTART_FILE);
-        }
-        
-        public static string GetXdgAutoStartDir()
-        {
-
-            string xdgConfigDir = Environment.GetEnvironmentVariable("XDG_CONFIG_HOME");
-
-            if (xdgConfigDir == null)
-                xdgConfigDir = String.Format(Constants.XDG_CONFIG_HOME,
-                                             Environment.GetFolderPath(Environment.SpecialFolder.Personal));
-
-            if (!xdgConfigDir.EndsWith("/"))
-                xdgConfigDir = String.Concat(xdgConfigDir,"/");
-            
-            /* assuming that the user has full access to xdgConfigDir */
-            string autostartDir = String.Concat(xdgConfigDir,Constants.XDG_AUTOSTART_DIR);    
-            if(!Directory.Exists(autostartDir))
-                Directory.CreateDirectory(autostartDir);
-                
-            return autostartDir;
-            
-        }
-        
-        public static void ShowCannotPublishServiceDialog()
-        {
-
-            using (Gtk.MessageDialog errorDialog = 
-                    new Gtk.MessageDialog(  null,
-                                            DialogFlags.Modal,
-                                            MessageType.Error,
-                                            ButtonsType.Close,
-                                            Catalog.GetString("Failed to publish the file sharing service on the network.\nPlease ensure that the Avahi service is running.")))
-            {
-                errorDialog.WindowPosition = WindowPosition.Center;
-                errorDialog.Run();
-                errorDialog.Destroy();
-                
-                while (Gtk.Application.EventsPending ())
-                    Gtk.Application.RunIteration ();                
-
-            }
-
-        }
-        
-        public static void ShowGenericFatalErrorDialog()
-        {
-        
-            using (Gtk.MessageDialog errorDialog = 
-                    new Gtk.MessageDialog(  null,
-                                            DialogFlags.Modal,
-                                            MessageType.Error,
-                                            ButtonsType.Close,
-                                            Catalog.GetString("Failed to start file sharing service.")))
-            {
-                errorDialog.WindowPosition = WindowPosition.Center;
-                errorDialog.Run();
-                errorDialog.Destroy();
-
-                while (Gtk.Application.EventsPending ())
-                    Gtk.Application.RunIteration ();                
-
-            }
-
-        }
-
-        public static void InvokeLater(EventHandler code)
-        {
-            Thread thread = new Thread(new ThreadStart(code));
-            thread.IsBackground = true;
-            thread.Start();
-        }
-        
-        public delegate void BackgroundTask();
-        
-        public static void InvokeLater(BackgroundTask code)
-        {
-            Thread thread = new Thread(new ThreadStart(code));
-            thread.IsBackground = true;
-            thread.Start();
-        }
-    }
-    
 }
