@@ -1525,8 +1525,9 @@ namespace GShare.FTP
 
         public FTPServer(   IAuthenticationManager auth,
                             IPermissionManager permissions,
-                            string rootDir)
-            : base(IPAddress.Any,0)
+                            string rootDir,
+                            int serverPort)
+            : base(IPAddress.Any,serverPort)
         {
             iConnectedClients = new Hashtable();
             iAuthentication = auth;
